package fr.itsonus.bousoleplussbackend.controllers;

import fr.itsonus.bousoleplussbackend.domain.quiz.model.Bundle;
import fr.itsonus.bousoleplussbackend.domain.quiz.model.Question;
import fr.itsonus.bousoleplussbackend.payload.request.BundleCreationRequest;
import fr.itsonus.bousoleplussbackend.usecase.BundleUseCase;
import jakarta.validation.Valid;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Slf4j
@RestController
@RequestMapping("/bundles")
@AllArgsConstructor
public class BundleController {

    private final BundleUseCase bundleUseCase;

    @PostMapping
    public Bundle create(@Valid @RequestBody BundleCreationRequest request) {
        return bundleUseCase.create(request.label(), request.questions().stream().map(BundleCreationRequest.QuestionCreationRequest::toQuestion).toList());
    }

    @GetMapping
    public List<Bundle> findAllForCurrentUser() {
        return bundleUseCase.findAllForCurrentUser();
    }

    @GetMapping("{id}")
    public Bundle findById(@PathVariable("id") Long id) {
        return bundleUseCase.findById(id);
    }

    @GetMapping("{id}/questions/search")
    public Iterable<Question> findQuestions(@PathVariable Long id, @RequestParam Long axeId) {
        return bundleUseCase.findQuestions(id, axeId);
    }
}
