package fr.itsonus.bousoleplussbackend.infrastructure.postgres.repositories;

import fr.itsonus.bousoleplussbackend.domain.quiz.model.Quiz;
import fr.itsonus.bousoleplussbackend.domain.quiz.model.QuizResponse;
import fr.itsonus.bousoleplussbackend.domain.quiz.spi.ResponseCacheRepository;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.models.PostgresQuizResponseDto;
import fr.itsonus.bousoleplussbackend.payload.request.ResponseRequest;
import jakarta.transaction.Transactional;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.NoSuchElementException;
import java.util.Set;

@Service
@AllArgsConstructor
public class ResponseCacheProxyRepository implements ResponseCacheRepository {

    private final ResponseCachePostgresRepository responseCachePostgresRepository;
    private final QuestionCachePostgresRepository questionCachePostgresRepository;
    private final QuizCachePostgresRepository quizCachePostgresRepository;

    @Transactional
    public void saveQuizzResponses(Long bundleId, Quiz quiz, Set<ResponseRequest> responses) {
        var quizDto = quizCachePostgresRepository.findById(quiz.getId()).orElseThrow();
        responses.forEach(response -> {
            var question = questionCachePostgresRepository.findByIdAndBundleId(response.getQuestionId(), bundleId)
                    .orElseThrow(() -> new NoSuchElementException("No such question with id " + response.getQuestionId()));
            responseCachePostgresRepository.save(
                    new PostgresQuizResponseDto()
                            .setScore(response.getScore())
                            .setComment(response.getComment())
                            .setQuiz(quizDto)
                            .setQuestion(question));
        });
    }

    public List<QuizResponse> findAllByQuizId(Long quizId) {
        return responseCachePostgresRepository.findAllByQuizIdOrderByQuestionIndexAsc(quizId)
                .stream().map(PostgresQuizResponseDto::toQuizResponse).toList();
    }
}
